/*
===============================================================================

	FILE:	slider.cpp
	
	CONTENTS:
	
		Code for a simple slider in OpenGL
	
	PROGRAMMERS:
	
		Scott Cooper, derived from the GLOW library written by
		Daniel Azuma (DA)  <dazuma@kagi.com>
		Martin Isenburg, simplified from Scott Coopers hack
	
	COPYRIGHT:
	
		Copyright (C) 2000  Scott Cooper (scooper@cs.unc.edu)
		Copyright (C) 2000  Martin Isenburg (isenburg@cs.unc.edu)
		
		This library is free software; you can redistribute it and/or
		modify it under the terms of the GNU Lesser General Public
		License as published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option) any later version.
		
		This library is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
		GNU Lesser General Public License for more details.
		
		You should have received a copy of the GNU Lesser General Public
		License along with this library; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
	
	CHANGE HISTORY:
	
		27 April 2000 -- SC Hacked mercilessly for Martin's
				    SIGGRAPH submission.
		28 April 2000 -- MI Hacked more merciless all night.
	
===============================================================================
*/

#include <math.h>
#include <stdio.h>
#include <GL/glut.h>

#include "slider.h"

// YOU MUST PROVIDE THIS FUNCTION
extern int WindowHeight();

static float stripColor[3] = {0.5f, 0.5f, 0.5f};
static float indicatorColor[3] = {0.7f, 0.7f, 0.7f};
static float tickMarkColor[3] = {0.0f, 0.0f, 0.0f};
static float hiliteIndicatorColor[3] = {0.8f, 0.6f, 0.6f};
static float disableStripColor[3] = {0.7f, 0.7f, 0.7f};
static float disableIndicatorColor[3] = {0.7f, 0.7f, 0.7f};
static float disableTickMarkColor[3] = {0.3f, 0.3f, 0.3f};
static float disableOutlineColor[3] = {0.3f, 0.3f, 0.3f};
static float lightBevelColor[3] = {1.0f, 1.0f, 1.0f};
static float darkBevelColor[3] = {0.3f, 0.3f, 0.3f};

static void RightBevelColor() { ::glColor3fv(darkBevelColor); }
static void LeftBevelColor() { ::glColor3fv(lightBevelColor); }
static void TopBevelColor() { ::glColor3fv(darkBevelColor); }
static void BottomBevelColor() { ::glColor3fv(lightBevelColor); }

TUISlider::TUISlider(const int x, const int y, const int width, const int height)
{
  fXPosition = x;
  fYPosition = y;
  fWidth = width;
  fHeight = height;
  
  fMin = 0.0;
  fMax = 1.0;
  fNumTicks = 0;
  
  fValue = 0;
  fDragging = 0;
  fXOffset = 0;
  fYOffset = 0;

  fTop = fYPosition;
  fLeft = fXPosition;
  fBottom = fYPosition+fHeight;
  fRight = fXPosition+fWidth;
}

TUISlider::TUISlider(const int x, const int y, const int width, const int height, const float min, const float max )
{
  fXPosition = x;
  fYPosition = y;
  fWidth = width;
  fHeight = height;
  
  fMin = min;
  fMax = max;
  fNumTicks = 0;
  
  fValue = 0;
  fDragging = 0;
  fXOffset = 0;
  fYOffset = 0;
}

void TUISlider::SetMin(const float min)
{
  fMin = min;
}

void TUISlider::SetMax(const float max)
{
  fMax = max;
}
  
void TUISlider::SetValue(const float value)
{
  fValue = value;
}
  
int TUISlider::IsInside(int x, int y)
{  
   if ( (y > fTop) && (y < fBottom) && (x > fLeft) && (x < fRight) )
     {
       return 1;
     }
   else
     {
       return 0;
     } 
}

int TUISlider::IsDragging()
{
  return fDragging;
}

void TUISlider::OnMouseDown(int button, int x, int y)
{
  x = x - fLeft;
  y = y - fTop;

  int indicRadius, indicPos;
  float valRatio;
 
  valRatio = (fValue-fMin)/(fMax-fMin);

  if (Width() > Height())
    {
      indicRadius = Height()/2;
      indicPos = indicRadius+static_cast<int>(float(Width()-2*indicRadius)*valRatio);
      if (x > indicPos-indicRadius && x < indicPos+indicRadius)
	{
	  fXOffset = indicPos-x;
	  fYOffset = 0;
	  fDragging = 1;
	}
    }
  else
    {
      indicRadius = Width()/2;
      indicPos = indicRadius+static_cast<int>(float(Height()-2*indicRadius)*valRatio);
      if (y > indicPos-indicRadius && y < indicPos+indicRadius)
	{
	  fXOffset = 0;
	  fYOffset = indicPos-y;
	  fDragging = 1;
	}
    }
}

void TUISlider::OnMouseUp(int button, int x, int y)
{
  x = x - fLeft;
  y = y - fTop;

  if (fDragging)
    {
      fDragging = 0;
      CalcValue(x+fXOffset, y+fYOffset);
      OnReleased();
    }
}

void TUISlider::OnMouseMove(int x, int y)
{	
  x = x - fLeft;
  y = y - fTop;

  if (fDragging)
    {
      CalcValue(x+fXOffset, y+fYOffset);
      OnDragged();
    }
}

void TUISlider::OnReleased()
{
}

void TUISlider::OnDragged()
{
}

void TUISlider::CalcValue(int x, int y)
{
  if (Width() > Height())
    {
      fValue = float(x-Height()/2)/float(Width()-Height());
    }
  else
    {
      fValue = float(y-Width()/2)/float(Height()-Width());
    }
  if (fValue > 1.0f) fValue = 1.0f;
  if (fValue < 0.0f) fValue = 0.0f;
  
  fValue = fMin*(1.0f-fValue)+fMax*fValue;
}

void TUISlider::OnPaint()
{
  float pseudoWidth;
  float pseudoHeight;

#if 0
  if ((_type & ticksOnTop) != 0)
    {
      ::glScalef(-1.0f, 1.0f, 1.0f);
    }
  if ((_type & decreasing) == 0)
    {
      ::glScalef(1.0f, -1.0f, 1.0f);
    }
#endif
  if (Width() < Height())
    {
      ::glScalef(1.0f, -1.0f, 1.0f);
      ::glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
      pseudoWidth = Height();
      pseudoHeight = Width();
    }
  else
    {
      ::glScalef(1.0f, -1.0f, 1.0f);
      pseudoWidth = Width();
      pseudoHeight = Height();
    }
	
  GLfloat bevelHeight = float(4)/pseudoHeight;
  GLfloat bevelWidth = float(4)/pseudoWidth;
  GLfloat pixelHeight = float(2)/pseudoHeight;
  GLfloat pixelWidth = float(2)/pseudoWidth;
	
  // Draw strip
  float top, bottom;
  if (fNumTicks == 0)
    {
      top = 0.25f;
      bottom = -0.25f;
    }
  else
    {
      top = 0.4f;
      bottom = -0.1f;
    }
	
  // Strip back
  if (!IsActive())
    {
      ::glColor3fv(disableStripColor);
    }
  else
    {
      ::glColor3fv(stripColor);
    }
  ::glRectf(-1.0f, bottom, 1.0f, top);

  if (IsActive())
    {
      // Left bevel
      RightBevelColor();
      ::glBegin(GL_POLYGON);
      ::glVertex2f(-1.0f, bottom);
      ::glVertex2f(-1.0f+bevelWidth, bottom+bevelHeight);
      ::glVertex2f(-1.0f+bevelWidth, top-bevelHeight);
      ::glVertex2f(-1.0f, top);
      ::glEnd();
		
      // Top bevel
      BottomBevelColor();
      ::glBegin(GL_POLYGON);
      ::glVertex2f(-1.0f, top);
      ::glVertex2f(-1.0f+bevelWidth, top-bevelHeight);
      ::glVertex2f(1.0f-bevelWidth, top-bevelHeight);
      ::glVertex2f(1.0f, top);
      ::glEnd();
		
      // Right bevel
      LeftBevelColor();
      ::glBegin(GL_POLYGON);
      ::glVertex2f(1.0f, bottom);
      ::glVertex2f(1.0f-bevelWidth, bottom+bevelHeight);
      ::glVertex2f(1.0f-bevelWidth, top-bevelHeight);
      ::glVertex2f(1.0f, top);
      ::glEnd();
		
      // Bottom bevel
      TopBevelColor();
      ::glBegin(GL_POLYGON);
      ::glVertex2f(-1.0f, bottom);
      ::glVertex2f(-1.0f+bevelWidth, bottom+bevelHeight);
      ::glVertex2f(1.0f-bevelWidth, bottom+bevelHeight);
      ::glVertex2f(1.0f, bottom);
      ::glEnd();
    }
  else
    {
      ::glColor3fv(disableOutlineColor);
      ::glBegin(GL_LINE_LOOP);
      ::glVertex2f(-1.0f, bottom);
      ::glVertex2f(-1.0f, top);
      ::glVertex2f(1.0f, top);
      ::glVertex2f(1.0f, bottom);
      ::glEnd();
    }
	
  // Draw tickmarks if necessary
  float indicRadius = pseudoHeight/pseudoWidth;
  if (!IsActive())
    {
      ::glColor3fv(disableTickMarkColor);
    }
  else
    {
      ::glColor3fv(tickMarkColor);
    }
  ::glLineWidth(2);
  if (fNumTicks == 1)
    {
      ::glBegin(GL_LINES);
      ::glVertex2f(0.0f, -0.5f);
      ::glVertex2f(0.0f, -1.0f);
      ::glEnd();
    }
  else if (fNumTicks > 1)
    {
      for (int i=0; i<fNumTicks; i++)
	{
	  float pos = float(i)/float(fNumTicks-1)*2.0f-1.0f;
	  pos *= (1.0-indicRadius);
	  ::glBegin(GL_LINES);
	  ::glVertex2f(pos, -0.5f);
	  ::glVertex2f(pos, -1.0f);
	  ::glEnd();
	}
    }
  ::glLineWidth(1);

  // Draw indicator
  float indicPos;
  indicPos = (fValue-fMin)/(fMax-fMin);
  indicPos = indicRadius-1.0f+indicPos*
    (pseudoWidth-pseudoHeight)/pseudoWidth*2.0f;
  if (fNumTicks > 0)
    {
      // With ticks (pentagons)
      if (!IsActive())
	{
	  ::glColor3fv(disableIndicatorColor);
	}
      else if (fDragging)
	{
	  ::glColor3fv(hiliteIndicatorColor);
	}
      else
	{
	  ::glColor3fv(indicatorColor);
	}
      ::glBegin(GL_POLYGON);
      ::glVertex2f(indicPos, -0.5f);
      ::glVertex2f(indicPos-indicRadius, 0.0f);
      ::glVertex2f(indicPos-indicRadius, 1.0f);
      ::glVertex2f(indicPos+indicRadius, 1.0f);
      ::glVertex2f(indicPos+indicRadius, 0.0f);
      ::glEnd();
		
      if (IsActive())
	{
	  // Bottom (crooked) bevel
	  BottomBevelColor();
	  ::glBegin(GL_QUAD_STRIP);
	  ::glVertex2f(indicPos-indicRadius, 0.0f);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, bevelHeight);
	  ::glVertex2f(indicPos, -0.5f);
	  ::glVertex2f(indicPos, -0.5f+6.0f/pseudoHeight);
	  ::glVertex2f(indicPos+indicRadius, 0.0f);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, bevelHeight);
	  ::glEnd();
			
	  // Left bevel
	  LeftBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos-indicRadius, 0.0f);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, bevelHeight);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, 1.0f-bevelHeight);
	  ::glVertex2f(indicPos-indicRadius, 1.0f);
	  ::glEnd();
			
	  // Top bevel
	  TopBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos-indicRadius, 1.0f);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, 1.0f-bevelHeight);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, 1.0f-bevelHeight);
	  ::glVertex2f(indicPos+indicRadius, 1.0f);
	  ::glEnd();
			
	  // Right bevel
	  RightBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos+indicRadius, 0.0f);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, bevelHeight);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, 1.0f-bevelHeight);
	  ::glVertex2f(indicPos+indicRadius, 1.0f);
	  ::glEnd();
			
	  // Ridges
	  float ridgeTop = 1.0f-bevelHeight*2.0f;
	  float ridgeBottom = -0.5f+bevelHeight*3.0f;
	  LeftBevelColor();
	  ::glBegin(GL_LINES);
	  ::glVertex2f(indicPos-float(7)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos-float(7)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos-float(1)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos-float(1)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos+float(5)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos+float(5)/pseudoWidth, ridgeTop);
	  ::glEnd();
	  RightBevelColor();
	  ::glBegin(GL_LINES);
	  ::glVertex2f(indicPos-float(5)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos-float(5)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos+float(1)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos+float(1)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos+float(7)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos+float(7)/pseudoWidth, ridgeTop);
	  ::glEnd();
	}
      else
	{
	  ::glColor3fv(disableOutlineColor);
	  ::glBegin(GL_LINE_LOOP);
	  ::glVertex2f(indicPos, -0.5f);
	  ::glVertex2f(indicPos-indicRadius+pixelWidth, 0.0f);
	  ::glVertex2f(indicPos-indicRadius+pixelWidth, 1.0f-pixelHeight);
	  ::glVertex2f(indicPos+indicRadius-pixelWidth, 1.0f-pixelHeight);
	  ::glVertex2f(indicPos+indicRadius-pixelWidth, 0.0f);
	  ::glEnd();
	}
    }
  else
    {
      // No ticks (rects)
      if (!IsActive())
	{
	  glColor3fv(disableIndicatorColor);
	}
      else if (fDragging)
	{
	  glColor3fv(hiliteIndicatorColor);
	}
      else
	{
	  glColor3fv(indicatorColor);
	}
      ::glRectf(indicPos-indicRadius, -1.0f, indicPos+indicRadius, 1.0f);
		
      if (IsActive())
	{
	  // Bottom bevel
	  BottomBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos-indicRadius, -1.0);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, -1.0+bevelHeight);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, -1.0+bevelHeight);
	  ::glVertex2f(indicPos+indicRadius, -1.0);
	  ::glEnd();
			
	  // Left bevel
	  LeftBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos-indicRadius, -1.0);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, -1.0+bevelHeight);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, 1.0-bevelHeight);
	  ::glVertex2f(indicPos-indicRadius, 1.0);
	  ::glEnd();
			
	  // Top bevel
	  TopBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos-indicRadius, 1.0);
	  ::glVertex2f(indicPos-indicRadius+bevelWidth, 1.0-bevelHeight);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, 1.0-bevelHeight);
	  ::glVertex2f(indicPos+indicRadius, 1.0);
	  ::glEnd();
			
	  // Right bevel
	  RightBevelColor();
	  ::glBegin(GL_QUADS);
	  ::glVertex2f(indicPos+indicRadius, -1.0);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, -1.0+bevelHeight);
	  ::glVertex2f(indicPos+indicRadius-bevelWidth, 1.0-bevelHeight);
	  ::glVertex2f(indicPos+indicRadius, 1.0);
	  ::glEnd();
			
	  // Ridges
	  float ridgeTop = 1.0-bevelHeight*2.0;
	  float ridgeBottom = -1.0+bevelHeight*2.0;
	  LeftBevelColor();
	  ::glBegin(GL_LINES);
	  ::glVertex2f(indicPos-float(7)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos-float(7)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos-float(1)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos-float(1)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos+float(5)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos+float(5)/pseudoWidth, ridgeTop);
	  ::glEnd();
	  RightBevelColor();
	  ::glBegin(GL_LINES);
	  ::glVertex2f(indicPos-float(5)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos-float(5)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos+float(1)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos+float(1)/pseudoWidth, ridgeTop);
	  ::glVertex2f(indicPos+float(7)/pseudoWidth, ridgeBottom);
	  ::glVertex2f(indicPos+float(7)/pseudoWidth, ridgeTop);
	  ::glEnd();
	}
      else
	{
	  glColor3fv(disableOutlineColor);
	  ::glBegin(GL_LINE_LOOP);
	  ::glVertex2f(indicPos-indicRadius, -1.0f);
	  ::glVertex2f(indicPos-indicRadius, 1.0f);
	  ::glVertex2f(indicPos+indicRadius, 1.0f);
	  ::glVertex2f(indicPos+indicRadius, -1.0f);
	  ::glEnd();
	}
    }
}

void TUISlider::DoDisplay()
{
   	::glViewport(fXPosition, WindowHeight()-fYPosition-Height(), Width(), Height());
	::glMatrixMode(GL_PROJECTION);
	::glLoadIdentity();
	::glMatrixMode(GL_MODELVIEW);
	::glLoadIdentity();
	if (1)
	{
		::glGetIntegerv(GL_SCISSOR_BOX, fOldScissor);
		int x = fXPosition;
		if (x < fOldScissor[0]) x = fOldScissor[0];
		int y = WindowHeight()-fYPosition-Height();
		if (y < fOldScissor[1]) y = fOldScissor[1];
		int w = Width();
		if (x+w>fOldScissor[0]+fOldScissor[2]) w = fOldScissor[0]+fOldScissor[2]-x;
		int h = Height();
		if (y+h>fOldScissor[1]+fOldScissor[3]) h = fOldScissor[1]+fOldScissor[3]-y;
		::glScissor(x, y, w, h);
	}
	
	OnPaint();
	
	::glScissor(fOldScissor[0], fOldScissor[1], fOldScissor[2], fOldScissor[3]);

}
